#####################
# Econometrics SzDF #
# Theme 5 - IRF     #
#####################
rm(list=ls())
source("libs.R")
source("utils.R")

# 1. Understanding Impulse Response Function

Kmax   <- 39
IRFtab <- data.frame(H = 0:Kmax,IRF = rep(0,Kmax+1))

ggplot(IRFtab, aes(x=H)) + 
  geom_line(aes(y=IRF), colour="blue", size=2) + 
  theme_light()+
  theme(text = element_text(size=12))+
  labs(title="Impulse response function", y="", x="horizon", caption="")+
  geom_hline(yintercept=0)

MyPlot(IRFtab)

############################
# a. AR(1) model           #
# y[t] = rho*y[t-1] + e[t] #
# IRFk = rho^k             #
############################

rho    <- 0.8
IRF    <- rho^seq(0,Kmax,1)
IRFtab$IRF <- IRF

MyPlot(IRFtab)

####################################
# b. AR(1) with constant           #
# y[t] = alpha + rho*y[t-1] + e[t] #
# IRFk = rho^k                     # 
####################################

alpha  <- 2
rho    <- 0.8
IRF    <- rho^seq(0,Kmax,1)
IRFtab$IRF <- IRF

MyPlot(IRFtab)

# equilibrium level
mu <- alpha/ (1-rho)      

# simulation
nobs    = 100
ysim    = rep(0,nobs)
ysim[1] = mu
for(t in 2:nobs){
  ysim[t]=alpha+rho*ysim[t-1]+rnorm(1)
}

YsimTab <- data.frame(obs = 1:nobs,ysim = ysim)

MyPlot(YsimTab,main="AR(1) simulation", xlab="observation",hline=mu)+
  geom_hline(yintercept=mu, col="red", size=2) 
  # geom_hline(yintercept=mean(ysim), col="green", size=2)

###########################################
# c. AR(2) model                          #
# y[t] = rho1*y[t-1] + rho2*y[t-2] + e[t] #
###########################################

rho   <- c(1.3,-0.4)
IRF[1] = 1
IRF[2] = rho[1]
for(k in 3:(Kmax+1)){
  IRF[k] <- rho[1]*IRF[k-1]+rho[2]*IRF[k-2]
}

IRFtab$IRF <- IRF
MyPlot(IRFtab)


###########################################
# d. ARMA(1,1) model                      #
# y[t] = rho*y[t-1] + e[t] + thet*e[t-1]  #
###########################################

rho   <-  0.8
thet  <- -0.5
IRF0  <- rho^seq(0,Kmax,1)
IRF1  <- c(0,head(IRF0*thet[1],-1))
IRF   <- IRF0 + IRF1 

IRFtab$IRF <- IRF
MyPlot(IRFtab)

IRF[1:5]

help(ARMAtoMA) # function in stats package that allows to convert ARMA into MA
ARMAtoMA(ar = rho, ma = thet, 5)


####################################
# e. Random walk                   #
# y[t] = y[t-1] + e[t]             #
# IRFk = 1                         # 
####################################

rho    <- 1
IRF    <- rho^seq(0,Kmax,1)
IRFtab$IRF <- IRF
MyPlot(IRFtab)

# simulation
nobs    = 100
RWsim    = matrix(0,nobs,4)
for(k in 1:4){
  for(t in 2:nobs){
      RWsim[t,k]=rho*RWsim[t-1,k]+rnorm(1)
  }
}

RWsimTab <- data.frame(obs = 1:nobs, RWsim)
ggplot(RWsimTab, aes(x=obs)) + 
  geom_line(aes(y=X1),size=1, col=1)+
  geom_line(aes(y=X2),size=1, col=2)+
  geom_line(aes(y=X3),size=1, col=3)+
  geom_line(aes(y=X4),size=1, col=4)+
  theme_light()+
  theme(text = element_text(size=12))+
  labs(title="Random walk simulation", y="", x="observation", caption="")+
  geom_hline(yintercept=0, size=2) 


##########################
# f. VAR(1) model        #
# y[t] = A*y[t-1] + e[t] #
##########################

rho   <- c(1.3,-0.4)
A     <- matrix(c(rho, 1, 0),2,2,byrow=TRUE) # canonical matrix
temp  <- diag(1,2)
for(k in 1:(Kmax+1)){
  IRF[k] <- temp[1,1]
  temp   <- A%*%temp
}

IRFtab$IRF <- IRF
MyPlot(IRFtab)

##############################################
# Additional material                        #
# Forward looking models                     #
# g. y[t] = bet*Ey[t+1] + e[t] + thet*e[t-1] #
##############################################

bet      <- 0.8
thet     <- 0.5

#    v[t] = e[t]+thet*e[t-1]
#    y[t] = bet^0*v[t] + bet^1*E(v[t+1]) + bet^2*E(v[t+2])+...
# FIRF[k] = bet^k      -- discounting factor
# FCTX[k] = E(v[t+k])  -- expected value of external variable
#    y[t] = sum_(k=0)^infty FIRF[k]*FCTX[k]

FIRF    <- bet^seq(0,Kmax,1) 

# forecast for v[t+k] after a shock e[t]=1, i.e. E(v[t+k])
FCTX    <- rep(0,2*(Kmax+1))
FCTX[1] <- 1
FCTX[2] <- thet

# Calculating IRF, i.e how y[t+k] responds to e[t]
# y[t]   = IRF[0]*e[t] + IRF[1]e[t-1] + IRF[2]e[t-2]+....
# IRF[k] = sum_(s=0)^infty FIRF[k]*FCTX[k+s]
IRF    <- rep(NaN,Kmax)
for(k in 1:(Kmax+1)){
  IRF[k] = t(FIRF) %*% FCTX[k:(k+Kmax)]
}

IRFtab$IRF <- IRF
MyPlot(IRFtab)

#####################################
# h. Two equation model             #  
# p[t] = bet*E(p[t+1]) + kappa y[t] #
# y[t] = rho*y[t-1] + e[t]          # 
#####################################

bet    <- 1
rho    <- 0.8
kappa  <- 0.1

# y[t]    = e[t] + rho*e[t-1] + rho^2*e[t-2]+...
# v[t]    = kappa*y[t]
# p[t]    = bet^0 v[t] + bet^1*E(v[t+1]) + FIRF[2]*E(v[t+2])+....
# FIRF[k] = bet^k      -- discounting factor
# FCTX[k] = E(v[t+k])  -- expected value of external variable
#    p[t] = sum_(k=0)^infty FIRF[k]*FCTX[k]

FIRF    <- bet^seq(0,Kmax,1) 

# Forecast for v[t+k] after a shock e[t]=1, i.e. E(v[t+k])
FCTX    <- rho^seq(0,(2*Kmax+1),1)
FCTX    <- kappa*FCTX

# IRF
IRF    <- rep(NaN,Kmax)
for(k in 1:(Kmax+1)){
  IRF[k] = t(FIRF)%*%FCTX[k:(k+Kmax)]
}

IRFtab$IRF <- IRF
MyPlot(IRFtab)


###########################################
# j. Hybrid models models                 #
# y[t] = a1*Ey[t+1] + a2*y[t-1] + e[t]  #
###########################################

a1  <- 0.4
a2  <- 0.4

# Other notation
#              (-a2L +1 - a1L^(-1)) * y[t] = e[t]
# kappa (1 - rho L) (1- bet L^(-1)) * y[t] = e[t]

Roots <- Re(polyroot(c(-a2,1,-a1)))
# these are solution for L, hence the solution for L^(-1) is the inverse

kappa <- 0.8        # how to calculate kappa?
rho   <- Roots[1]   # autoregressive factor
bet   <- 1/Roots[2] # discount factor

# Now we can transform the system to what we know

# z[t]  = (1- rho L)^(-1)*e[t] = e[t] + rho*e[t-1] + rho^2*e[t-2]+...
# v[t]  = 1 / kappa * z[t]
# y[t]  = bet^0 E[v[t]] + bet^1*E(v[t+1]) + FIRF[2]*E(v[t+2])+....

FIRF    <- bet^seq(0,Kmax,1)
FCTX    <- 1/kappa*rho^seq(0,2*Kmax,1)

# Calculate IRF o y[t+k] against e[t]
IRF    <- rep(NaN,Kmax)
for(k in 1:(Kmax+1)){
  IRF[k] = t(FIRF)%*%FCTX[k:(k+Kmax)]
}

IRFtab$IRF <- IRF
MyPlot(IRFtab)

# check
IRF[1]*0.4 + IRF[3]*0.4
IRF[2]



###################################
# k. Blanchard-Khan               #
# y[t] = bet*Ey[t+1] + kappa*z[t] #
# z[t] = rho*z[t-1]  + e[t]       #
###################################################
# bet*Ey[t+1] = y[t] - kappa*z[t]                 #
#      z[t+1] =          rho*z[t-1] + e[t+1]      # 
###################################################

K     = 100;
bet   = 1;
rho   = 0.8;
kappa = 0.1;

##################################
# A0*EX[t+1] = A1*X[t] +B0*e[t]  #
##################################
  
A0 = rbind(c(bet,0),c(0,1))
A1 = rbind(c(1,-kappa),c(0,rho))
B0 = rbind(0,1)

###########################
# EX[t+1] = AX[t] +Be[t]  #
###########################

A = solve(A0)%*%A1
B = solve(A0)%*%B0

Lambda   = diag(eigen(A)$values)
V        = eigen(A)$vectors

V1          = solve(V)
R           = V1%*%B
# Compare A to V%*%Lambda%*%V1

# Dividing matrix V1 into parts 
n = dim(Lambda)[1]  
k = sum(diag(Lambda)>=1)   # number of unstable roots
v11 = V1[1:k    ,1:k,    , drop=F]  
v12 = V1[1:k    ,(k+1):n , drop=F]
v21 = V1[(k+1):n,1:k     , drop=F]     
v22 = V1[(k+1):n,(k+1):n , drop=F]

# Unstable root condition
# v11*y[t]+v12*z[t]=0 --> y[t] = -inv(v11)*v12*z[t]
Mzy = -solve(v11)%*%v12

# Stable root condition
# tilde(z)[t] = v21*y[t]+v22*z[t] 
# tilde(z)[t] = (v21*Mzy + v22)*z[t]
Mzz = solve(v21%*%Mzy + v22)

# Dynamics for tilde(z)[t]
Lambda22  = Lambda[(k+1):n,(k+1):n, drop=F]
r2        = R[(k+1):n,1, drop=F]  

# Calculating IRF
IRFztilde = t(Lambda22^(0:Kmax) %*% r2)
IRFz      = Mzz%*%IRFztilde
IRFy      = Mzy%*%IRFz

IRFtab$IRF <- as.numeric(IRFz)
MyPlot(IRFtab)+
  labs(title="Backward looking variable - z")

IRFtab$IRF <- as.numeric(IRFy)
MyPlot(IRFtab)+
  labs(title="Forward looking variable - y")

# Compare the results with point h!

###########################################################
# Housework: use Blanchard-Khan to solve 3-eq NK model    #
# (a) pi[t] = bet*E(pi[t+1]) + kappa*y[t]                 #
# (b) y[t]  = E(y[t+1]) - sig^(-1)*(i[t]-E(pi[t+1])       #
# (c) i[t]  = gam*pi[t] + eps[t]                          #
# (d) eps[t]= rho*eps[t-1]+nu[t]                          #
# Hint: sbusitute (c) to (b) to eliminate i[t]            #
###########################################################

bet   = 0.99
kappa = 0.1
sig   = 5
gam   = 1.5 
rho   = 0.8











