#####################
# Econometrics SzD  #
# Theme 6 - ARMA    #
#####################
rm(list=ls())
source("libs.R")
source("utils.R")

# a. Loading data
####################

# filename <- "dataJIE.csv"
# filename <- "http://web.sgh.waw.pl/~mrubas/AdvEcon/data/dataJIE.csv"
# data     <- read.csv(filename, header=TRUE, sep=";", dec=",", row.names="daty")
# names(data)

filename <- "dataJIE.RData"
load(filename)
names(data)

# Legend		
# CA	current account (% GDP)	
# I	domestic interest rate (annual rate)	
# P	CPI QoQ (log growth rate) 	
# Y	GDP QoQ	
# YF	Foreign output QoQ	
# PF	Foreign prices QoQ	
# IF	Foreign interest rate (annual rate)	
# NER	Nominal Exchange Rate QoQ (increase stands for depreciation)	
# RER	Real Exchange Rate QoQ (increase stands for depreciation)	
#
# Countries:
# US  - United States
# EA  - euro area
# UK  - United Kingdom
# CAD - Canada
# AUS - Australia
#
# Source: Michele Ca' Zorzi, Marcin Kolasa and Michal Rubaszek: Exchange rate forecasting with DSGE models
# link: https://doi.org/10.1016/j.jinteco.2017.03.011

y     <- data["Y_US"]   
dates <- as.yearqtr(as.Date(rownames(data),format="%d.%m.%Y"))

x  <- zoo(y, dates)
x0 <- ts(y,start=c(1975,1), freq=4)
plot(x)

par(mfrow=c(2,2), cex = 0.7, bty="l")
plot(x, main="time series", sub="", xlab="", ylab=""); abline(h=mean(x),col="red")
hist(x, main="histogram", ylab="", sub="", xlab="")
Acf(x0, main="ACF", ylab="", sub="", xlab="")
Pacf(x0, 12, main="PACF", ylab ="", sub="", xlab="")

# b. Ad-hoc ARMA(2,0) model
##############################

dev.off()
temp = Arima(x0, order = c(2,0,0))
summary(temp)
plot(temp)

# c. Choosing specification, information criteria
###################################################

LagSel <- function(x, Pmax=4, Qmax=4, d=0, crit="BIC"){
  IC <- matrix(NA, Pmax+1, Qmax+1)
  for(p in 0:Pmax){
    if(p==Pmax){x0=x} else {x0 = x[-(1:(Pmax-p))]} 
  	for(q in 0:Qmax){
  		if(crit == "AIC"){ IC[p+1,q+1] <- AIC(Arima(x0,order=c(p,d,q)), k=2) }
  		if(crit == "BIC"){ IC[p+1,q+1] <- AIC(Arima(x0,order=c(p,d,q)), k=log(length(x)-p)) }
      if(crit == "HQ"){  IC[p+1,q+1] <- AIC(Arima(x[- (1:(Pmax-p+1))],order=c(p,d,q)), k=2*log(log(length(x)-p))) }
  }}
  rownames(IC) <- paste('ar',0:Pmax, sep=""); colnames(IC) <- paste('ma',0:Qmax, sep="")
  return(IC)
}

kable(LagSel(x0, crit="BIC"),digits=2) 
kable(LagSel(x0, crit="AIC"),digits=2) 

arma0 <- arima(x0, order = c(1,0,0))  # parsimonous model
arma1 <- arima(x0, order = c(2,0,1))  # larger model

# d. Choosing specification, Likelihood ratio test
###################################################

LLtest <- function(arimaS, arimaL){
	d  <-  -2*(arimaS[["loglik"]] - arimaL[["loglik"]])
	df <-  length(arimaL[["coef"]]) - length(arimaS[["coef"]])
  p  <- 1 - pchisq(d, df)
  return(list(stat=d, prob = p)) 
}
as.data.frame(LLtest(arma0, arma1))

# e. Choosing specification, Portmanteau test for autocorrelation
#################################################################

dev.off()
Acf(arma0$residuals, main="ACF of residuals")

k <- length(arma0[["coef"]])-1
Jmax <- 8
LB   <- matrix(NA,Jmax-k, 2)
for(h in (k+1):Jmax){
  z <- Box.test(residuals(arma0), lag = h, type = "Ljung-Box", fitdf = k)
  LB[h-k,] <- c(z[["statistic"]], z[["p.value"]])
}
rownames(LB) <- paste('h=',(k+1):Jmax, sep=""); colnames(LB) <- c("LB stat.", "prawd.")
kable(LB, digits=3)

# f. JB normality test
#######################

jarque.bera.test(residuals(arma0))

e         <- residuals(arma0)/sd(residuals(arma0))
bwdth     <- 0.1
ggplot(data.frame(e), aes(x = e)) +
  theme_light() +
  geom_histogram(binwidth = bwdth, colour = "white", fill = "blue", size = 0.1) +
  stat_function(fun = function(x) dnorm(x)*length(e)*bwdth, color = "red", size = 2) +
  labs(title="Empirical historgram vs normal distribution", y="", x="Standarized residuals", caption="")

# g. Characteristic roots 
##############################

ar.coef <- arma0[["coef"]][1]
# ma.coef <- arma0[["coef"]][2:3]
ma.coef = NULL

roots=polyroot(c(1,-ar.coef))
abs(1/roots)

plot(arma0)

# i. MA representation
##############################

Kmax = 39

IRF    <- ARMAtoMA(ar = ar.coef,ma = ma.coef,Kmax)
IRF    <- c(1,IRF)
IRFtab <- data.frame(H = 0:Kmax,IRF = IRF)
MyPlot(IRFtab, main="IRF from ARMA")

# what about the larger ARMA(3,2) model

IRF1        <- ARMAtoMA(ar = arma1[["coef"]][1:2],ma = arma1[["coef"]][3],Kmax)
IRFtab$IRF1 <- c(1,IRF1)

MyPlot(IRFtab, main="IRF from ARMA again")+
  geom_line(aes(y=IRF1),size=1, col=2)


# j. Forecast                   
####################################
H = 16

fcst <- forecast(arma0, h=H)
head(fcst)
plot(fcst, include=16, shaded=T, shadecols=c('grey','GhostWhite'), main="inflation forecast", ylab="", xlab="", bty="l")
abline(h=mean(x),col="red")
abline(h=arma0$coef["intercept"],col="green")


# forecast path
datesf <- last(index(x)) + (1:H)/4
temp     <- zoo(fcst$mean,datesf)
xf = rbind(x,temp)

kable(temp,digits=2)



