#####################
# Econometrics SzD  #
# Theme 7 - VAR     #
#####################
rm(list=ls())
source("libs.R")
source("utils.R")

# a. Loading data
####################

# filename <- "dataJIE.csv"
# filename <- "http://web.sgh.waw.pl/~mrubas/AdvEcon/data/dataJIE.csv"
# data     <- read.csv(filename, header=TRUE, sep=";", dec=",", row.names="daty")
# names(data)

filename <- "dataJIE.RData"
load(filename)
names(data)

x     <- cbind(data["Y_US"],data["Y_UK"])   
dates <- as.yearqtr(as.Date(rownames(data),format="%d.%m.%Y"))

z  <- zoo(x, dates)
y  <- ts(x,start=c(1975,1), freq=4)
plot(y)

dev.off()
plot(z, screens=c(1,1),col=1:2,main="model variables", type = "l", ylab="",xlab="", bty="l")
legend("bottomleft",legend=names(z),col=1:2,lty=1, bty="n")

# b. VAR model              
#############################

VARselect(z, lag.max=6, type="const")
p    = 2
VARE <- VAR(z, p=p, type="const")

summary(VARE)
Acoef(VARE)
Bcoef(VARE)

# Testing for autocorrelation: Ljung-Boxa
serial.test(VARE, lags.pt = 4, type="PT.adjusted")

par(mfrow=c(2,2), cex = 0.7, bty="l", mar=c(3,3,3,3))
Acf(residuals(VARE)[,1], xlab="",main="e1 vs lags of e1")
Ccf(residuals(VARE)[,1],residuals(VARE)[,2],xlab="",main="e1 vs lags of e2", xlim=c(0,24))
Acf(residuals(VARE)[,2],xlab="",main="e2 vs lags of e2")
Ccf(residuals(VARE)[,2],residuals(VARE)[,1],xlab="",main="e2 vs lags of e1", xlim=c(0,24))

# c. causality
################################

causality(VARE, cause="Y_US")
causality(VARE, cause="Y_UK")

# d. recursive structuralization 
#################################

bmat <- matrix(0,2,2)
bmat[lower.tri(bmat, diag=T)] <- NA
SVARE <- SVAR(VARE, Amat = NULL, Bmat = bmat)
SVARE
B <- SVARE$B

# e. structural IRF 
#####################
K    <- 16
SVMA <- Phi(SVARE, nstep=K)

# short run-impact matrix
SVMA[,,1]

# Plot in Vars package (not nice)
SIRF <- irf(SVARE, n.ahead=K, cum = F, boot = F)
plot(SIRF)

# Single IRF
IRFtab = data.frame(H=0:K,IRF=SVMA[1,1,])
MyPlot(IRFtab, main= "example IRF")

# Panel of IRFs  
IRFtab$IRF = SVMA[1,1,]; irf11 <- MyPlot(IRFtab, main = "eps US --> GDP US")
IRFtab$IRF = SVMA[1,2,]; irf12 <- MyPlot(IRFtab, main = "eps UK --> GDP US")
IRFtab$IRF = SVMA[2,1,]; irf21 <- MyPlot(IRFtab, main = "eps US --> GDP UK")
IRFtab$IRF = SVMA[2,2,]; irf22 <- MyPlot(IRFtab, main = "eps UK --> GDP UK")
grid.arrange(irf11,irf12,irf21,irf22, ncol=2)

# where do we see these values?
B

# f. FEVD           
#####################

i      = 2                    # which variables
SIRFi  = SVMA[i,,]            # IRF for variable i
temp   = SIRFi^2              # contribution of shock[t] to forecast variance at [t+h]
FVARi  = apply(temp,1,cumsum) # contribution of shocks[t:t+h] to forecast variance at [t+h]
FEVD0  <- 100*prop.table(FVARi,1); colnames(FEVD0) = c("uUS","uUK")
FEVD1  <- data.frame(H = 0:K, epsUS = FEVD0[,1], epsUK = FEVD0[,2])

kable(FEVD1[c(1:4,8,12,16),],digits=3,row.names = FALSE)

FEVD2 <- gather(data = FEVD1, key = shocks, value = value, -c(H))
ggplot(FEVD2, aes(fill=shocks, y=value, x=H)) + 
   geom_bar( stat="identity") +   
   theme_light()+
   labs(title="Forecast Error Variance Decomposition", y="", x="", caption="")+
   scale_fill_manual(values=c("grey70", "grey30"))

# function in the Vars package (not so nice as Ggplot)
temp <- fevd(SVARE,n.ahead=K)
temp$Y_UK
plot(temp)

# g. Historical decomposition   #
#################################

i      = 2            # which variables
# --> Structural shocks
e <- residuals(VARE)
B <- SVARE$B
u <- t(solve(B)%*%t(e))
T <- dim(u)[1]
# cor(u) 

# --> IRFs
SVMA <- Phi(SVARE, nstep=T)
SIRF  = t(SVMA[i,,])    # IRF for variable i

# --> Historical decompositoion
HistDec             <- matrix(NA,T,2)
colnames(HistDec)   <- c("epsUS","epsUK")

for(t in 1:T){
  junk1 <- as.matrix(u[1:t,])
  junk2 <- as.matrix(SIRF[t:1,])
  HistDec[t,] <- colSums(junk1*junk2)
}

tail(cbind(HistDec[,1]+HistDec[,2],tail(coredata(z[,i]),T)))
tail(HistDec[,1]+HistDec[,2]-tail(coredata(z[,i]),T))

# --> initial conditions
InitCond <- tail(coredata(z[,i]),T) - colSums(t(HistDec)) 
mu       <- as.numeric(tail(InitCond,1))
InitCond <- InitCond - mu
HistDec  <- cbind(HistDec, InitCond)

# --> plot
HD1   <- as.data.frame(HistDec); HD1$t   <- as.Date(rownames(HistDec),format="%d.%m.%Y") 
HD2   <- gather(data = HD1, key = shocks, value = value, -c(t))
Y     <- data.frame(y = tail(coredata(z[,i]),T)-mu,t=as.Date(rownames(HistDec),format="%d.%m.%Y"))
 
ggplot() + 
   geom_bar(data=HD2, aes(fill=shocks, y=value, x=t), stat="identity",width=50) + # 
   scale_fill_manual(values=c("red", "green","grey80"))+
   geom_line(data=Y, aes(y=y, x=t), size=1)+
   theme_light()+
   labs(title="Historical decomposition for GDP growth in UK", y="", x="", caption="")

# h. Forecast 
#################

H   <- 16
i   <- 2         # variable number
T   <- dim(z)[1]

fct <- predict(VARE, n.ahead=H)
fct
fanchart(fct, nc=1, xlim=c(T-16,T+H),bty="l",mar=c(3,3,1,1))

# fan chart in ggplot
datesf <- last(index(z))+(1:H)/4
f      <- zoo(fct$fcst[[i]][,1],datesf)

hist_data  = data.frame(t=index(z),y = coredata(z[,i]))
fct_data   = data.frame(t=datesf  ,y = fct$fcst[[i]][,1]) 
point_fct = rbind(hist_data,fct_data)
interval_fct = data.frame(t=datesf,lower = fct$fcst[[i]][,2],upper = fct$fcst[[i]][,3]) 
MyPlot(point_fct,main="Forecast from VAR model",xlab="",hline=mean(z[,i]))+
   geom_ribbon(data= interval_fct, aes(x=t,ymin=lower, ymax=upper), alpha=0.2)+
   xlim(2000,2018)+
   ylim(-3,3)

# forecast path
datesf <- last(index(z))+(1:H)/4
temp     <- zoo(fct$fcst[[i]][,1],datesf)
xf = rbind(z[,i],temp)

kable(temp,digits=2)

###################################################################################
# Long-run restrictions                                                           #
# 3-VAR model for Exchange Rate                                                   #
# Richard Clarida, Jordi Gali, 1994                                               #
# Sources of Real Exchange Rate Fluctuations: How Important are Nominal Shocks?   #
# NBER Working Paper No. 4658                                                     #
# http://www.nber.org/papers/w4658                                                #
###################################################################################

# In the long run:
# output driven by:      supply shocks
# real exchange rate by: supply + demand shocks
# price indices:         supply + demand + monetary shocks
# RER misalignment only due to monetary shocks


x     <- cbind(data["Y_US"]-data["YF_US"],data["RER_US"], data["P_US"]-data["PF_US"])     
dates <- as.yearqtr(as.Date(rownames(data),format="%d.%m.%Y"))

z  <- zoo(x, dates)
y  <- ts(x,start=c(1975,1), freq=4)
plot(y)

dev.off()
plot(z, screens=c(1,1,1),col=1:3,main="model variables", type = "l", ylab="",xlab="", bty="l")
legend("bottomleft",legend=names(z),col=1:3,lty=1, bty="n")

# To do
# a. Estimate VAR model (select lags, check for autocorrelation)
# b. Identify shocks (Long-run restrictions)
# c. Plot IRF for levels
# d. Plot FEVD for levels (you need to do it from cumulated IRF)
# e. Calculate historical decompositiion
# f. make a plot of RER misalignment + equilibrium RER

# a. Estimate VAR model (select lags, check for autocorrelation)
VARselect(y, lag.max = 4)
P     <- 3
varER <- VAR(y, P)
summary(varER)

# b. structuralization (LR restrictions)

svarER <- BQ(varER)
svarER

# c. Plot IRF for levels
nam <- c("output","RER","price")
Theta    <- Phi(svarER,40)
ThetaCum <- aperm(apply(Theta, 1:2, cumsum), c(2,3,1)) 
par(mfrow=c(3,3), cex = 0.5, bty="l")
for(res in 1:3){
  for(imp in 1:3){
    plot.ts(ThetaCum[res,imp,], ylab=nam[res], main=paste("response to",nam[imp], "shock"), lwd=2)
    abline(h=0)
  }
}

# compare for LR restrictions 

# d. Plot FEVD for levels (you need to do it from cumulated IRF)
dev.off()
nVar = 2
IRF  <- ThetaCum[nVar,,]
IRF2 <- IRF^2
junk <- apply(IRF2,1,cumsum)
FEVDour <- prop.table(junk, 1)
junk <- barplot(t(FEVDour),main="FEVD", legend=c("Prod","Demand","Monetary"), xlab="Horizon")
axis(1, at=junk,labels=1:41, col.axis="black", las=1)

# e. Calculate historical decompositiion

# structural shocks
u   <- resid(varER)
B   <- svarER$B
eta <- solve(B)%*%t(u)
T   <- dim(eta)[2]

# IRFs
nVar = 2
Theta <- Phi(svarER,T-1)
IRF   <- Theta[nVar,,]

# Decomposition
DecDQ           <- matrix(NA,3,T)
for(t in 1:T){
  junk1 <- as.matrix(eta[,1:t])
  junk2 <- as.matrix(IRF[,t:1])
  DecDQ[,t] <- rowSums(junk1*junk2)
}

# calculating initial conditions
InitCond <- (y[-(1:3),nVar] - colSums(DecDQ))
plot.ts(InitCond)

# plot of historical decomposition
DecDQ <- rbind(DecDQ, InitCond)
rownames(DecDQ) <- c(rownames(eta),"InitCond")

tmp <- barplot(DecDQ,main="Historical decomposition of RER Q/Q",
                legend=rownames(DecDQ), xlab="", col = c("red","green", "blue", "grey"))
lines(x=tmp, y=y[-(1:3),2], lwd=2)

# f. make a plot of RER misalignment + equilibrium RER
RERmis <- cumsum(DecDQ[3,])
RERmis <- RERmis - mean(RERmis)
barplot(t(RERmis),main="RER misalignment", col = c("red"))

RER    <- cumsum(y[-(1:P),2])
REReq  <- RER-RERmis
xnam   <- as.Date(rownames(data),format="%d.%m.%Y")[-(1:P)]
plot(xnam,RER, col="black", type="l",lwd=2, main = "Equilibrium Exchange Rate", xlab="")
lines(xnam,REReq, col="red", type="l",lwd=2)

# misalignment
barplot(RERmis, main="RER misalignment", names.arg=year(xnam))
