#################################
# Econometrics SzD              #
# Theme 9A: BAyesian regression #
#################################

rm(list=ls())
source("libs.R")
source("utils.R")

rm(list=ls())

# a. Loading data

# filename <- "dataJIE.csv"
# filename <- "http://web.sgh.waw.pl/~mrubas/AdvEcon/data/dataJIE.csv"
# data     <- read.csv(filename, header=TRUE, sep=";", dec=",", row.names="daty")
# names(data)

filename <- "dataJIE.RData"
load(filename)

data[,11:50] <- 4*data[,11:50]                            # annualization
dates        <- as.Date(rownames(data),format="%d.%m.%Y")

# b. Observation matrices

y <- as.matrix(data$I_US)
x <- as.matrix(cbind(1,data$P_US)); colnames(x)=c("const","inf")
T <- dim(x)[1]
K <- dim(x)[2]

# c. Function to caluclte moments
XYmoments <- function(y,x){
  out <- list()
  out$xtx          <- t(x)%*%x
  out$xtx.inv      <- chol2inv(chol(out$xtx))
  out$xty          <- t(x)%*%y
  out$yty          <- t(y)%*%y
  out$numsamp.data <- length(y)
  return(out)
}
out <- XYmoments(y,x) 

# d. ML estimates
###################################
betaML  <- out$xtx.inv%*%out$xty
vML     <- T - K
e       <- y - x%*%betaML
hML     <- as.numeric(t(e)%*%e/vML)
OmegaML <- hML*out$xtx.inv

# presentation of the results
temp <- cbind(betaML,sqrt(diag(OmegaML)),betaML/sqrt(diag(OmegaML)))
rownames(temp) <- colnames(x); colnames(temp) <- c("beta","sd","t-Student")
kable(temp,digits=2)
# compare to summary(lm(y~x-1))

# e. Conjugate prior, fixed variance 
####################################
beta0  <- c(0,1.5)                      # mean for beta
Omega0 <- diag(c(10,0.1))^2             # variance for neta0

# Posterior
Omega1 <- solve(solve(Omega0)+1/hML*out$xtx)
beta1  <- Omega1%*%(solve(Omega0)%*%beta0 + 1/hML*out$xty)
# Omega1 <- solve(solve(Omega0)+solve(OmegaML))
# beta1  <- Omega1%*%(solve(Omega0)%*%beta0 + solve(OmegaML)%*%betaML)

temp <- cbind(betaML,beta0,beta1)
colnames(temp) <- c("ML","Prior","Posterior")
kable(temp,digits=2)

k     = 2
grid  = seq(0,2,0.01)
prior     = dnorm(grid,beta0[k],Omega0[k,k]^0.5) 
LL        = dnorm(grid,betaML[k],OmegaML[k,k]^0.5) 
posterior = dnorm(grid,beta1[k],Omega1[k,k]^0.5) 
temp <- data.frame(cbind(grid,prior,LL,posterior))
colnames(temp) = c("ThetaGrid","prior","LL","posterior")
ggplot(data = temp, aes(x = ThetaGrid)) + 
  geom_line(aes(y = prior,colour="prior")) + 
  geom_line(aes(y = LL,colour="LL")) +
  geom_line(aes(y = posterior,colour="posterior")) +
  labs(title="Normal prior, linear regression", y="density function", x="parameter value", caption="")+
  theme_bw()+
  scale_colour_manual("", 
                      breaks = c("prior", "LL", "posterior"),
                      values = c("red", "green", "black")) 

# 4. Normal-(inverse)Gamma prior
##################################
v0     <- 2                         # usual values
h0     <- hML
beta0  <- c(0,1.5)                  # mean for beta
Omega0 <- diag(c(10,0.1))^2/h0      # variance for beta0 (scaled by h0)


grid = seq(0.01,20,0.01)
priorh <- dinvgamma(grid,v0,h0)
par(mfrow=c(1,1), cex = 0.7, bty="l")
plot(grid,priorh,type="l",main="distribution o variance parameter",xlab="",ylab="")

# posterior
Omega1 <- solve(solve(Omega0)+out$xtx)
beta1  <- Omega1%*%(solve(Omega0)%*%beta0 + out$xty)
v1     <- v0 + vML
h1     <- as.numeric(h0*v0 + vML*hML + t(beta0-betaML)%*%Omega1%*%(beta0-betaML))

posteriorh <- dinvgamma(grid,v1,h1)
temp <- data.frame(cbind(grid,priorh,posteriorh))
colnames(temp) = c("grid","prior","posterior")
ggplot(data = temp, aes(x = grid)) + 
  geom_line(aes(y = prior,colour="prior")) + 
  geom_line(aes(y = posterior,colour="posterior")) +
  labs(title="distribution for variance parameter", y="density function", x="parameter value", caption="")+
  theme_bw()+
  scale_colour_manual("", 
                      breaks = c("prior", "posterior"),
                      values = c("red", "black"))

# Gibbs sampling procedure
Nsim = 10000
beta_draws <- matrix(NA,Nsim,K)
h_draws    <- rep(NA,Nsim)

for(i in 1:Nsim){
  h_draws[i]    <- rinvgamma(1,v1,h1)
  beta_draws[i,] <- mvrnorm(1,beta1,h_draws[i]*Omega1)
}

bmean    <- colMeans(beta_draws)
hmean    <- mean(h_draws)
temp <- cbind(betaML,beta1,bmean)
colnames(temp) <- c("ML","Post 1","Post 2")
kable(temp,digits=2)

# 5. Similar exercise in BayesSummaryStatLM package
####################################################

# Prior options
beta.prior    = list(type = "mvnorm.known", mean.mu = beta0, cov.C = Omega0*h0)
sigmasq.prior = list("inverse.gamma", v0, h0, hML)

temp <- bayes.regress(data.values=out, 
                      beta.prior=beta.prior, 
                      sigmasq.prior=sigmasq.prior,
                      Tsamp.out=Nsim)


bmean1    <- colMeans(temp$beta)
hmean1    <- mean(temp$sigmasq)
bsd      <- apply(beta_draws,2,sd)
hmean    <- mean(h_draws)
temp <- cbind(betaML,betaML/sqrt(diag(OmegaML)), beta0,bmean,bmean/bsd)
colnames(temp) <- c("ML","t-Stat", "Prior","Posterior","t-stat")
kable(temp,digits=2)

# 6. Large dataset model
####################################################

data0 <- data[,c(1:25,41:45)] # we remove RER/foreign vars due to collinearity problem

i <- 10 # variable number (interest rate in US)
y <- as.matrix(data0[,i]); colnames(y) <- names(data0)[i]
x <- as.matrix(cbind(1,data0[,-i])); colnames(x)=c("const",names(data0)[-i])
T <- dim(x)[1]
K <- dim(x)[2]

summary(lm(y~x-1))

# ML estimates

out <- XYmoments(y,x) 
betaML  <- out$xtx.inv%*%out$xty
vML     <- T-K
e       <- y - x%*%betaML
hML     <- as.numeric(t(e)%*%e/vML)
OmegaML <- hML*out$xtx.inv

# prior
v0            <- 2
h0            <- hML
beta0         <- rep(0,K)             # mean for beta
beta0[15]     <- 1.5                  # coefficient for infl in US   
Omega0        <- diag(0.01^2,K)/h0    # variance for beta0 (scaled by h0)
Omega0[1,1]   <- 100^2/h0             # constant
names(beta0)  <- colnames(Omega0) <- rownames(Omega0)  <- colnames(x)

# customiaztion
beta0["P_US"]         <- 1.5          # coefficient for infl in US   
Omega0["P_US","P_US"] <- 100^2/h0     # for this coefficent we are more flexible

# posterior
Omega1 <- solve(solve(Omega0)+out$xtx)
beta1  <- Omega1%*%(solve(Omega0)%*%beta0 + out$xty)
v1     <- v0 + vML
h1     <- as.numeric(h0*v0 + vML*hML + t(beta0-betaML)%*%Omega1%*%(beta0-betaML))


# Gibbs sampling procedure
Nsim = 1000
beta_draws <- matrix(NA,Nsim,K)
h_draws    <- rep(NA,Nsim)

for(i in 1:Nsim){
  h_draws[i]    <- rinvgamma(1,v1,h1)
  beta_draws[i,] <- mvrnorm(1,beta1,h_draws[i]*Omega1)
}

bmean    <- colMeans(beta_draws)
bsd      <- apply(beta_draws,2,sd)
btstud   <- bmean/bsd
hmean    <- mean(h_draws)
temp <- cbind(betaML,betaML/sqrt(diag(OmegaML)), beta0,bmean,btstud)
colnames(temp) <- c("ML","t-Stat", "Prior","Posterior","t-stat")

kable(temp, digits=2)

# Main takeaway: 
# with Bayesian analysis we can handle nicely models with a large number of regressors!!!








