##########################################
# Econometrics SzD                       #
# Theme 9b - BVAR with Litterman prior   #
##########################################
rm(list=ls())
source("libs.R")
source("utils.R")

# a. Loading data
####################

# filename <- "dataJIE.csv"
# filename <- "http://web.sgh.waw.pl/~mrubas/AdvEcon/data/dataJIE.csv"
# data     <- read.csv(filename, header=TRUE, sep=";", dec=",", row.names="daty")
# names(data)

filename <- "dataJIE.RData"
load(filename)
names(data)

######################
# Small, 2-VAR BVAR  #
######################
# growth rates
xgrowth   <- cbind(data["Y_US"],data["Y_UK"])   
dates     <- as.yearqtr(as.Date(rownames(data),format="%d.%m.%Y"))

# changing into log-levels
xlevel  <- apply(xgrowth,2,cumsum)/100
xzoo    <- zoo(xlevel, dates)
z       <- ts(xlevel,start=c(1975,1), freq=4)
plot(z)

dev.off()
plot(xzoo, screens=c(1,1),col=1:2,main="model variables", type = "l", ylab="",xlab="", bty="l")
legend("bottomleft",legend=names(xzoo),col=1:2,lty=1, bty="n")

#############################
# step by step calculations #
# see. Bear Toolbox p. 25-29#
#############################

P      <- 2                  # maximulm lag
temp   <- VARmat(z,P)        # matrixes y and x
y<- temp$y; x<- temp$x
cbind(head(y),head(x))
K <- ncol(y)                  
T <- nrow(y)
L <- ncol(x)

# variance of residuals from AR(P) models (we will need this later)
Sig2AR  <- apply(z,2,function(x) arima(x, c(P,0,0))$sigma2)
SigAR   <- Sig2AR^0.5

# ML estimates
Ahat    <- solve(t(x)%*%x)%*%t(x)%*%y
res     <- y - x%*%Ahat
# For Sighat there are three options
# Option 1: non-diagonal, on the basis of residuals from VAR(P). We take into account the correlations. See Bear Toolbox 
Sighat  <- 1/T*t(res)%*%res
# Option 2: diagonal, on the basis of residuals from VAR(P). We treat each equation independently. Original Litterman
# Sighat  <- diag(diag(1/T*t(res)%*%res))
# Option 3: diagonal, on the basis of residuals from AR(P). Not frequently used
# Sighat  <- diag(Sig2AR)

# We can get here with the following function
# VARest <- VAR(z, p=P, type="const")
# t(Bcoef(VARest))
# Sighat <- 1/T*t(resid(VARest))%*%resid(VARest)

# setting the prior
lam1    <- 0.1   # overall tightness
lam2    <- 0.5   # cross-variable tightness
lam3    <- 1     # lag decay
lam4    <- 100   # constant tightness

# Prior mean for A
ARcoeff <- c(1,1)
Aprior  <- matrix(0,L,K); Aprior[2:(K+1),] <- diag(ARcoeff); 
colnames(Aprior)<-colnames(z); rownames(Aprior) <- colnames(x)

# Prior variance for A 
OmPrior<- matrix(0, L*K, L*K)
for(i in 1:K){
  OmPrior[(i-1)*L+1,(i-1)*L+1] = (lam1*lam4*SigAR[i]) ^ 2 # constant
  for(p in 1:P){
    for(j in 1:K){
      if(i==j){
        OmPrior[j+(p-1)*K +(i-1)*L+1,j+(p-1)*K+(i-1)*L+1]   = (lam1*p^(-lam3)*SigAR[i]/SigAR[j]) ^ 2
      } else {
        OmPrior[j+(p-1)*K +(i-1)*L+1,j+(p-1)*K + (i-1)*L+1] = (lam1*lam2*p^(-lam3)*SigAR[i]/SigAR[j]) ^ 2
      }
      
    }
  }
}

# calculating prosterior
OmPost <- solve(solve(OmPrior) +  kronecker(solve(Sighat),t(x)%*%x,"*"))
Apost  <- OmPost%*%(solve(OmPrior)%*% vec(Aprior) + kronecker(solve(Sighat),t(x),"*")%*%vec(y))

Apost <- matrix(Apost,L,K)
rownames(Apost)<-rownames(Aprior); colnames(Apost)<-colnames(Aprior)

kable(cbind(Ahat,Aprior,Apost),digits=3)
# play with hyperparameters and look for Apost 

# Sampling
Ndraws <- 1000
Adraws <- array(0, dim = c(L, K, Ndraws))
for(n in 1:Ndraws){
  temp         <- mvrnorm(n = 1, mu = vec(Apost), Sigma = OmPost)
  Adraws[,,n]  <- matrix(temp,L,K)
}


# For each draw we can compute IRF, forecast
Amean    <- Apost
Amean[,] <- apply(Adraws,c(1,2),mean)
Amean

kable(cbind(Amean,Apost),digits=3)

# How much draws do we need to get Amean close to Apost?

# Bayesian IRF
################################

H      <- 12

# IRF for each draw from the posterior
BAYESirf <- array(NaN, dim=c(K,K,H,Ndraws))
for (n in 1:Ndraws){
  BAYESirf[,,,n] = singleIRF(Adraws[,,n],Sighat,H)
}

# Plot IRFS for all draws
var_nam <- colnames(y)
sh_nam  <- c("eps_US","eps_UK")
par(mfrow=c(K,K), cex = 0.7, bty="l")
for(res in 1:K){
  for(imp in 1:K){
    y1 <- min(BAYESirf[res,imp,,]); y2<-max(BAYESirf[res,imp,,])
    plot.ts(BAYESirf[res,imp,,1], ylim = c(y1,y2))
    abline(h=0)
    for(k in 2:Ndraws){
      lines(BAYESirf[res,imp,,k])
    }
    medResp <- apply(BAYESirf[res,imp,,],1,median)
    lines(medResp,col="red",lwd=3)
  }
}

# A nicer plot
CI = 0.9           # confidence interval
figs <- list(); plN = 1
for(res in 1:2){
  for(imp in 1:2){
    irfDist <- 100*data.frame(t(apply(BAYESirf[res,imp,,],1,quantile, probs=c((1-CI)/2,0.5,1-(1-CI)/2))))
    irfDist <- cbind(0:(H-1),irfDist)    
    names(irfDist) <- c("horizon","lower","median","upper")
    figs[[plN]] <- ggplot(irfDist, aes(x=horizon)) + 
      geom_line(aes(y=median), colour="blue") + 
      geom_ribbon(aes(ymin=lower, ymax=upper), alpha=0.2) +
      theme_minimal()+
      labs(title=paste(sh_nam[imp],"shock"), y=var_nam[res], x="horizon", caption="")+
      geom_hline(yintercept=0)
    plN = plN+1  
  }
}
grid.arrange(figs[[1]],figs[[2]],figs[[3]], figs[[4]],ncol=2)


# Bayesian Foreacast
###########################
H      <- 12 # forecast horizon
Hlast  <- 10 # last observations on the graph   

BAYESfct <- array(NaN, dim=c(H,K,Ndraws))
for (n in 1:Ndraws){
  BAYESfct[,,n] = singleFCT(Adraws[,,n],Sighat,H,y,shock=TRUE)
}

# Plot for draws from predictive denity
var_nam <- colnames(y)
par(mfrow=c(1,K), cex = 0.7, bty="l")
for(i in 1:K){
  ylast <- tail(y[,i],Hlast) 
  y1    <- min(BAYESfct[,i,],ylast); y2<-max(BAYESfct[,i,],ylast)
  plot.ts(c(ylast,BAYESfct[,i,1]), ylim = c(y1,y2),col="grey",ylab="",xlab="",main=var_nam[i])
  for(k in 2:Ndraws){
    lines(c(ylast,BAYESfct[,i,k]),col="grey")
  }
  medResp <- apply(BAYESfct[,i,],1,median)
  lines(c(ylast,medResp),col="black",lwd=3)
}

# A nicer plot
CI = 0.9           # confidence interval
figs <- list(); plN = 1
for(i in 1:2){
  ylast   <- tail(y[,i],Hlast) 
  fctDist <- t(apply(BAYESfct[,i,],1,quantile, probs=c((1-CI)/2,0.5,1-(1-CI)/2)))
  fctDist <- data.frame(rbind(matrix(ylast,Hlast,3),fctDist))
  fctDist <- cbind((-Hlast+1):H,fctDist)    
  names(fctDist) <- c("horizon","lower","median","upper")
  figs[[plN]] <- ggplot(fctDist, aes(x=horizon)) + 
    geom_line(aes(y=median), colour="blue") + 
    geom_ribbon(aes(ymin=lower, ymax=upper), alpha=0.2) +
    theme_minimal()+
    labs(title=var_nam[i], y="", x="horizon", caption="")
  plN = plN+1  
}
grid.arrange(figs[[1]],figs[[2]],ncol=2)

