##########################################
# Econometrics SzD                       #
# Theme 9c - Normal-Wishart prior        #
##########################################
rm(list=ls())
source("libs.R")
source("utils.R")

# a. Loading data
####################

# filename <- "dataJIE.csv"
# filename <- "http://web.sgh.waw.pl/~mrubas/AdvEcon/data/dataJIE.csv"
# data     <- read.csv(filename, header=TRUE, sep=";", dec=",", row.names="daty")
# names(data)

filename <- "dataJIE.RData"
load(filename)
names(data)

######################
# Small, 2-VAR BVAR  #
######################
# growth rates
xgrowth   <- cbind(data["Y_US"],data["Y_UK"])   
dates     <- as.yearqtr(as.Date(rownames(data),format="%d.%m.%Y"))

# changing into log-levels
xlevel  <- apply(xgrowth,2,cumsum)/100
xzoo   <- zoo(xlevel, dates)
z       <- ts(xlevel,start=c(1975,1), freq=4)
plot(z)

dev.off()
plot(xzoo, screens=c(1,1),col=1:2,main="model variables", type = "l", ylab="",xlab="", bty="l")
legend("bottomleft",legend=names(xzoo),col=1:2,lty=1, bty="n")

#############################
# step by step calculations #
# see. Bear Toolbox p. 30-34#
#############################

P      <- 2                  # maximulm lag
temp   <- VARmat(z,P)        # matrixes y and x
y<- temp$y; x<- temp$x
cbind(head(y),head(x))
K <- ncol(y)                  
T <- nrow(y)
L <- ncol(x)

# variance of residuals from AR(P) models (we will need this later)
Sig2AR  <- apply(z,2,function(x) arima(x, c(P,0,0))$sigma2)
SigAR   <- Sig2AR^0.5

# ML estimates
Ahat    <- solve(t(x)%*%x)%*%t(x)%*%y
res     <- y - x%*%Ahat
Sighat  <- 1/T*t(res)%*%res

# setting the prior
lam1    <- 0.1   # overall tightness
lam3    <- 1     # lag decay
lam4    <- 100   # constant tightness

# Inverse Wishart Prior for Sigma
SigPrior   <- diag(Sig2AR)
vPrior     <- K+2

# Prior mean for A
ARcoeff <- c(1,1)
Aprior  <- matrix(0,L,K); Aprior[2:(K+1),] <- diag(ARcoeff); 
colnames(Aprior)<-colnames(z); rownames(Aprior)<-colnames(x)  

# variance 
Psi0    <- matrix(0,L,L)
for(j in 1:K){
  Psi0[1,1] = (lam4*lam1) ^ 2
  for(p in 1:P){
    Psi0[j+(p-1)*K+1,j+(p-1)*K+1] = (lam1*p^(-lam3)/SigAR[j]) ^ 2
  }
}
OmPrior <- kronecker(diag(Sig2AR), Psi0, FUN = "*") 

# Getting posterior matrices
Psi1    <- solve(solve(Psi0) + t(x)%*%x)
Apost   <- Psi1%*%(solve(Psi0)%*%Aprior + t(x)%*%y)
vPost   <- vPrior + T
SigPost <- t(y)%*%y + SigPrior + t(Aprior)%*%solve(Psi0)%*%Aprior - t(Apost)%*%solve(Psi1)%*%Apost

# Sampling
Ndraws <- 1000
Sigdraws <- array(0, dim = c(K, K, Ndraws)) 
Adraws   <- array(0, dim = c(L, K, Ndraws))

for(n in 1:Ndraws){
  Sigdraws[,,n] <- riwish(vPost, SigPost)
  junk          <- kronecker(Sigdraws[,,n], Psi1, FUN = "*") 
  temp          <- mvrnorm(n = 1, mu = vec(Apost), Sigma = junk)
  Adraws[,,n]   <- matrix(temp,L,K)
}


# For each draw we can compute IRF, forecast
Amean <- apply(Adraws,c(1,2),mean)
colnames(Amean)<- colnames(Apost); rownames(Amean)<- rownames(Apost)

kable(cbind(Ahat,Aprior,Amean),digits=3)
# play with hyperparameters and look for Apost 


# Bayesian IRF
################################

H      <- 12

# IRF for each draw from the posterior
BAYESirf <- array(NaN, dim=c(K,K,H,Ndraws))
for (n in 1:Ndraws){
  BAYESirf[,,,n] = singleIRF(Adraws[,,n],Sigdraws[,,n],H)
}

# Plot IRFS
var_nam <- colnames(y)
sh_nam  <- c("eps_US","eps_UK")
CI = 0.9           # confidence interval
figs <- list(); plN = 1
for(res in 1:2){
  for(imp in 1:2){
    irfDist <- 100*data.frame(t(apply(BAYESirf[res,imp,,],1,quantile, probs=c((1-CI)/2,0.5,1-(1-CI)/2))))
    irfDist <- cbind(0:(H-1),irfDist)    
    names(irfDist) <- c("horizon","lower","median","upper")
    figs[[plN]] <- ggplot(irfDist, aes(x=horizon)) + 
      geom_line(aes(y=median), colour="blue") + 
      geom_ribbon(aes(ymin=lower, ymax=upper), alpha=0.2) +
      theme_minimal()+
      labs(title=paste(sh_nam[imp],"shock"), y=var_nam[res], x="horizon", caption="")+
      geom_hline(yintercept=0)
    plN = plN+1  
  }
}
grid.arrange(figs[[1]],figs[[2]],figs[[3]], figs[[4]],ncol=2)


# Bayesian Foreacast
###########################
H      <- 12 # forecast horizon
Hlast  <- 10 # last observations on the graph   

BAYESfct <- array(NaN, dim=c(H,K,Ndraws))
for (n in 1:Ndraws){
  BAYESfct[,,n] = singleFCT(Adraws[,,n],Sigdraws[,,n],H,y,shock=TRUE)
}

# Plot interval forecast
var_nam <- colnames(y)
CI = 0.9           # confidence interval
figs <- list(); plN = 1
for(i in 1:2){
  ylast   <- tail(y[,i],Hlast) 
  fctDist <- t(apply(BAYESfct[,i,],1,quantile, probs=c((1-CI)/2,0.5,1-(1-CI)/2)))
  fctDist <- data.frame(rbind(matrix(ylast,Hlast,3),fctDist))
  fctDist <- cbind((-Hlast+1):H,fctDist)    
  names(fctDist) <- c("horizon","lower","median","upper")
  figs[[plN]] <- ggplot(fctDist, aes(x=horizon)) + 
    geom_line(aes(y=median), colour="blue") + 
    geom_ribbon(aes(ymin=lower, ymax=upper), alpha=0.2) +
    theme_minimal()+
    labs(title=var_nam[i], y="", x="horizon", caption="")
  plN = plN+1  
}
grid.arrange(figs[[1]],figs[[2]],ncol=2)
